function raymodel1(p,prodX0mat,beta,prodG,Dx,Dg,Ds,prodSJ,opt,pn1,namefileout,cutstep,cutpoint)
%
% Simulation model of ray growth and patterning - can simulate growth of
% one or multiple rays. 
% Used to generate results presented in Rolland-Lagan et al. (2012) Development 139: 1188-1197.
% User-defined input parameters:
% p: number of simulation steps
% prodX0mat: column vector of initial production rate values for X in the initial compartment
%            each value corresponds to alpha x0 for one ray (see equation 4b''' in the manuscript)
%            The size of X0 determines the number of rays
%            e.g. prodX0mat=[5 6 7 6 5 4 3 2 1 1 2 3 4 5 6 7 6 5]' for 18
%            rays, or prodX0mat=5 for the single ray model.
% beta: rate of increase of production rate for X (same as beta in paper), beta is the same size as prodX0mat
% prodG: production rate for G - corresponds to alpha g in equation 3b in te manuscript.
% Dx: Diffusion rate for X (see equation 2 in manuscript)
% Dg: Diffusion rate for G (see equation 1 in manuscript)
% Ds: Diffusion rate for S (see equation 6c in manuscript)
% prodSJ: component of the production rate of S depending on X at joints (alpha h in manuscript, see equation 6a)
% opt: option for model output: opt=1: make movie output, opt=2: save data
%      for further analysis, as well as figure outputs for a few time points (specified within the program).
% pn1: folder where model output will be stored.
% namefileout: recognizable name part for file outputs (movie, figure, and
%              data files); e.g. 'wildtype1'
% cutstep: time step at which amputation occurs. To run a model with no
%          amputation, set cutstep parameter to -1.
% cutpoint: compartment number at which amputation occurs (all compartments with larger compartment numbers are removed)
%           parameter value is irrelevant if cutstep is set to -1, but by
%           default we used a value of 0 in non-amputation simulations.
% example:
% raymodel1(100000,prodX0mat,beta,1,0.9,0.3,2,1,pn0,'wt2mov_wtG1cut20_Xprodlin',50000,15);
% e.g. single ray model with default parameters used in the paper, cut at step 50000 at 15 compartments:
% raymodel1(100000,5,0.005,1,0.9,0.3,2,1,pn0,'wt2mov_wtG1cut20_Xprodlin',50000,15);
%
%  (c) Anne-Galle Rolland-Lagan, University of Ottawa, 2011.


%Set of parameters which were not set as user defined as they remained the
%same for all simulations presented in the paper.
spacing=0.7; % parameter to space rays from one another in display output.
nrays=size(prodX0mat,1);%the number of rows in prodX0mat is used to calculate the number of rays present in the simulation

cpnthresh=100; %threshold parameter for creating output movie: a picture frame will be added to the movie output file
% every cpnthresh steps.
dt=0.01; %time interval between two simulation steps.

rw=nrays; % number of rows in matrix making up a grid of compartment (1 row per ray)
cl=1;% number of column in matrix making up the grid of compartments (the simulation starts with 1 compartment per ray)

thresh=0;  % threshold for concentration of X in last compartment, above which G is produced. This feature
% is not used in the manuscript as the value is set to 0.
gthresh=0;% threshold for growth to occur in response to G: compartments with sufficient level of G grow.
% feature is not used in the manuscript as the threshold is set
% to 0.
threshj=0.1;%threshold for S concentration below which a joint can form (sigma in the manuscript).
gamma=0.1;%threshold for g concentration below which a joint can form (gamma in the manuscript).

gcoeff=0.05; %rate determining compartment division level as a function of current level and G (alpha c in
%manuscript, see equation 5)

decs=0.8; % decay rate for S
decg=0.5;  % decay rate for G
decx=0.01;  % decay rate for X


prodXmax=100;%saturating value for prodX (X production rate), alpha x max in the paper. 
%Note: In simulations presented in the paper and all others run so far, the saturating value is never reached.
%at the end of each simulation, if saturation value has been reached a message will indicate this.
prodXmax_reached=0; % if prodX reaches prodXmax, this will take value 1.

% Initializing matrices of G, X, S concentrations, joint locations (J
% matrix), and compartment division level measure Cl (corresponds to C in the manuscript).
G=zeros(nrays,cl);
X=zeros(nrays,cl);
S=zeros(nrays,cl)+0;
J=zeros(nrays,cl);
Cl=ones(nrays,cl); %initial division level for each compartment = 1 (C0 value in manuscript)

%initialize X production rates prodX
prodX=prodX0mat;

%initialize length of ray prior to amputation
precutL=0;

%To make movie
nbf=20; %nb of frames per second
cd(pn1);
if opt==1,
    fna=namefileout;
    aviobj=avifile([fna '.avi'],'FPS',nbf,'Compression','Cinepak');
end;

%Initialize figure
figure
axis([-1 cl+1 -1 rw+1])
axis equal;
hold on, 

%initialize counter used in cunjunction with cpnthresh for movie frame
%capture
cpn=0;

% Initialize variables that record model output for further analysis with
% model_analysis program - only applies to single ray data
curtime=0; %current time
curveL=[];
curvesegnum=[];
Xrec=[];  %X concentration at the ray base
Grec=[];  %G concentration at the ray base
Srec=[];  %S concentration at the ray base
curveL=[];
segL=[];



%Initialize simulation step counter and boolean variables
i=0;%simulation step
cuthap=0;%0 if cut has not happened
stop=0;%0 if the simulation should not stop

clnbrays=ones(rw,1);%stores number of compartments in each ray - initialized to 1 per ray.

%loop time steps
while stop==0, 
    i=i+1; % update simulation step
    cpn=cpn+1; %update counter
    if cpn==cpnthresh+1,cpn=1;end; %counter reset if relevant
    
    %loop rays
    for ray=1:rw
        
        %calculate fluxes for all mobile substances
        clray=clnbrays(ray);
        
        flrix=zeros(1,clray);
        fllex=zeros(1,clray);
        flrig=zeros(1,clray);
        flleg=zeros(1,clray);
        flris=zeros(1,clray);
        flles=zeros(1,clray);
        
        flrix=Dx.*(X(ray,2:clray)-X(ray,1:clray-1));
        flrix=cat(2,flrix,0);
        fllex=-Dx*(X(ray,2:clray)-X(ray,1:clray-1));
        fllex=cat(2,0,fllex);
        fltotx=flrix+fllex;
        
        flrig=Dg.*(G(ray,2:clray)-G(ray,1:clray-1));
        flrig=cat(2,flrig,0);
        flleg=-Dg*(G(ray,2:clray)-G(ray,1:clray-1));
        flleg=cat(2,0,flleg);
        fltotg=flrig+flleg;
        
        flris=Ds.*(S(ray,2:clray)-S(ray,1:clray-1));
        flris=cat(2,flris,0);
        flles=-Ds*(S(ray,2:clray)-S(ray,1:clray-1));
        flles=cat(2,0,flles);
        fltots=flris+flles;
        
        %diffusion and decay of X
        X(ray,1:clray)=X(ray,1:clray)+(fltotx - decx*X(ray,1:clray))*dt;
        %production of X at ray base (in 1st compartment)
        if prodX(ray)<prodXmax
            prodX(ray)=prodX(ray)+beta(ray)*dt; % first update production rate, if it has not reached saturating value
        elseif prodXmax_reached==0
            prodXmax_reached=1;
            disp('prodX value has reached prodXmax value and is now constant - this will stabilize X concentration at the fin base');
        end
        X(ray,1)=X(ray,1)+prodX(ray)*dt; % then update X concentration at ray base
        %(concentration cannot fall below 0):
        indx= X(ray)<0;
        X(ray,indx)=0;
        
        %diffusion and decay of G
        G(ray,1:clray)=G(ray,1:clray)+(fltotg - decg*G(ray,1:clray))*dt;
        %(concentration cannot fall below 0):
        indx= G(ray,1:clray)<0;
        G(ray,indx)=0;
        
        %production of G (in distal compartment) depends on X
        if X(ray,clray)>thresh
            G(ray,clray)=G(ray,clray)+(prodG*X(ray,clray)*dt);
        end
        
        %diffusion and decay of S
        S(ray,1:clray)=S(ray,1:clray)+(fltots - decs*S(ray,1:clray))*dt;
        %(concentration cannot fall below 0):
        indx= S(ray,1:clray)<0;
        S(ray,indx)=0;
        %S production at joints
        indx=find(J(ray,:)==1); % find joints first
        S(ray,indx)=S(ray,indx)+prodSJ.*X(ray,indx).*dt; % S production at joints
        
        %making joint
        indx= S(ray,1:clray)<threshj & G(ray,1:clray)<gamma; %joint forms where S and G are low enough
        J(ray,indx)=1; %joint forms: value in J set to 1
        
        %cells with sufficient level of G grow (we have set that sufficient level gthresh to 0 for the manuscript)
        indx=find(G(ray,1:clray)>gthresh);
        Cl(ray,indx)=Cl(ray,indx)+ gcoeff*Cl(ray,indx).*G(ray,indx)*dt;
        indx=find(Cl(ray,1:clray)>2); % 'compartment' that have doubled Cl value ('division level') divide
        
        %if some compartments divide, all matrices have to be updated
        %accordingly:
        if ~isempty(indx)
            Cnew=[];Xnew=[];Gnew=[];Snew=[];Jnew=[];
            for kk=1:clray
                if find(indx==kk);
                    Cnew=cat(2,Cnew,Cl(ray,kk)/2,Cl(ray,kk)/2);
                    Xnew=cat(2,Xnew,X(ray,kk)/2,X(ray,kk)/2);
                    Gnew=cat(2,Gnew,G(ray,kk)/2,G(ray,kk)/2);
                    Snew=cat(2,Snew,S(ray,kk)/2,S(ray,kk)/2);
                    Jnew=cat(2,Jnew,J(ray,kk),0);% if there is a division at a joint compartment,
                    %the most proximal gets the joint
                    %note: the case where J(ray,kk)=1 should not happen often- add
                    %warning when this occurs
                    if J(ray,kk)==1
                        disp('a compartment with a joint is splitting');
                    end
                else
                    Cnew=cat(2,Cnew,Cl(ray,kk));
                    Xnew=cat(2,Xnew,X(ray,kk));
                    Gnew=cat(2,Gnew,G(ray,kk));
                    Snew=cat(2,Snew,S(ray,kk));
                    Jnew=cat(2,Jnew,J(ray,kk));
                end
                
            end
            
            %update nb of compartments for current ray:
            clray=size(Cnew,2);
            clnbrays(ray)=clray;
            %padd with zeros so that matrix has same number of entries on each row:
            %
            if size(Cnew,2)<size(Cl,2)
                %add the necessay number of zeros to the ray to match main matrix
                ntoadd=size(Cl,2)-size(Cnew,2);
                Cnew=cat(2,Cnew,zeros(1,ntoadd));
                Xnew=cat(2,Xnew,zeros(1,ntoadd));
                Gnew=cat(2,Gnew,zeros(1,ntoadd));
                Snew=cat(2,Snew,zeros(1,ntoadd));
                Jnew=cat(2,Jnew,zeros(1,ntoadd));
            elseif size(Cnew,2)>size(Cl,2)
                ntoadd=size(Cnew,2)-size(Cl,2);
                %other rows need to be padded with zeros
                Cl=cat(2,Cl,zeros(size(Cl,1),ntoadd));
                X=cat(2,X,zeros(size(X,1),ntoadd));
                G=cat(2,G,zeros(size(G,1),ntoadd));
                S=cat(2,S,zeros(size(S,1),ntoadd));
                J=cat(2,J,zeros(size(J,1),ntoadd));
            end
            
            Cl(ray,:)=Cnew;
            X(ray,:)=Xnew;
            G(ray,:)=Gnew;
            S(ray,:)=Snew;
            J(ray,:)=Jnew;
            
        end % end of loop that deals with grows (comparment division) 
    end % end of ray loop 
    
    curtime=curtime+dt; % update current time
    finmaxcl=max(clnbrays);%calculate fin length = length of longest ray of fin
    
    
     
    %If the fin has been amputated and its fin length regained the
    %preamputation length, or if there was no amputation but the maximal number of
    %simulation steps has been reached, the simulation will stop (variable stop set to 1)
    if (cuthap==1 && finmaxcl==precutL) || (cuthap==0 && i==p)
        stop=1;
    end
    
    %display model output,
    drawraysfig=0;
    %if this is the last step of the simulation, or a display step for movie or figures creation. 
    if stop==1 || ...
            (opt==1 && (cpn==cpnthresh || i==1 || i==cutstep || i==cutstep+1)) || ...
            (opt==2 &&(i==1 || i==1000 || i==10000 || i==50000 || i==55000 || i==65000 || i==75000 || i==cutstep || i==cutstep+1)),
        drawraysfig=1;
        cla;
        %call subprogram for ray display without joints
        draw_rays(X,G,max(max(X)),max(max(G)),nrays,clnbrays,spacing);%
        hold on,
                
        %for single ray model: output includes concentration curves for G,
        %X, and S
        if nrays==1
            h=line(1:clnbrays,X);set(h,'Color',[0 1 1]);
            h=line(1:clnbrays,G);set(h,'Color',[1 0 1]);
            h=line(1:clnbrays,S);set(h,'Color',[0 1 0]);
            h2=line(0:clnbrays,zeros(1,clnbrays+1));
            set(h2,'Color',[0 0 0]);
        end
        
        msp=0;
        
        %draw joints
        for rr=1:nrays
            indx=find(J(rr,:)==1); 
            if ~isempty(indx)
                for cp=1:size(indx,2)
                    h3=line([indx(cp);indx(cp)],[rr-0.5;rr+0.5]+msp);%is offset for drawing
                    set(h3,'LineWidth',2,'Color',[0 0 0]);
                end
                if stop==1;
                    distJ=cat(2,indx(1,1),indx(1,2:length(indx))-indx(1,1:length(indx)-1));
                    if nrays==1 % record variable for output analysis of single ray model only
                        segL=[indx(1,1:length(indx))' distJ'];
                    end
                end
            end

            msp=msp+spacing;
        end
    end
        
    %recording data for single ray analysis
    if nrays==1 && (cpn==cpnthresh || i==1 || i==cutstep || i==cutstep+1)
        curveL=cat(1,curveL,[curtime finmaxcl]);
        segnum=sum(J,2)+1;
        curvesegnum=cat(1,curvesegnum,[curtime segnum']);
        %record level of X in 1st (base) and last (distal) compartment over time
        Xrec=cat(1,Xrec,[curtime, X(1,1), X(1,cl)]);
        %record level of G in 1st (base) and last (distal) compartment over time
        Grec=cat(1,Grec,[curtime, G(1,1), G(1,cl)]);
        %record level of S in 1st (base) and last (distal) compartment over time
        Srec=cat(1,Srec,[curtime, S(1,1), S(1,cl)]);
        
        
        
    end;
    
    
    %update movie file or save figures
    if drawraysfig==1
        set(gca,'Color',[1 1 1]);
        set(gcf,'Color',[1 1 1]);
        axis equal;
        axis on;
        axis([0 60 -3 40]); %note: in the simulations for the manuscript we do not go beyond 60 compartments
        if opt==1
            set(gcf,'Position',[360 500 420 315])
            axis off;
            set(gcf, 'Renderer', 'zbuffer');
            frame=gcf;
            aviobj=addframe(aviobj,frame);
        elseif opt==2
            figsave=fullfile(pn1,[namefileout 'fig' num2str(i) '.fig']);
            saveas(gcf,figsave);
        end
        
    end
    
    %perform amputation if relevant
    if i==cutstep
        precutL=max(clnbrays);
        cl=cutpoint;%last compartment we keep, all next compartments are removed due to amputation
        
        clnbrays=zeros(size(clnbrays,1),1)+cl;
        %reset all necessary variables
        S=S(:,1:cl);
        X=X(:,1:cl);
        G=G(:,1:cl);
        Cl=Cl(:,1:cl);
        J=J(:,1:cl);
        cuthap=1;
    end
    
end %end of simulation steps loop (while loop)
   
%close movie file    
if opt==1,
    aviobj=close(aviobj);
    close all;
end;

%output variables for further analysis of single ray model
if nrays==1
    params=[thresh threshj prodX prodG prodSJ decs decg decx gthresh gcoeff Dx Dg Ds gamma]';
    fn2=fullfile(pn1,[namefileout 'data.mat']);
    save(fn2,'params','curveL','segL','curvesegnum','Xrec','Grec','Srec');
end


%%%%%%%%%%%% sub functions %%%%%%%%%%%
function draw_rays(s,g,thrs,thrg,rw,cl,spacing)
%subfunction used by raymodel1.m to create visual output of simulated rays
mat1=(1:60)./60;
matri=flipud([mat1',zeros(60,1)+1,zeros(60,1)+1]);
matrib=flipud([zeros(60,1)+1,mat1',zeros(60,1)+1]);

msp=0;
for m=1:rw,
    
    for n=1:cl(m),%because cl stores number of compartments for all rays in a column vector 
        
        xcoord=[n-0.5, n+0.5, n+0.5, n-0.5];
        ycoord=[m-0.5 m-0.5 m+0.5 m+0.5]+msp;
        
        cc=s(m,n);
        cg=g(m,n);

        if cc>0,
            if cc<thrs,
                colbr1=(matri((floor(((cc-0)/(thrs-0))*59 +1)),:)); 
            else
                colbr1=[0 1 1];
            end
        else
            colbr1=[1 1 1];
        end
        
        if cg>0,
            if cg<thrg,
                colbr2=(matrib((floor(((cg-0)/(thrg-0))*59 +1)),:)); 
            else
                colbr2=[1 0 1];
            end
        else
            colbr2=[1 1 1];
        end
        
        colbr=[colbr1;colbr2];
        colbrtot=[min(colbr(:,1)),min(colbr(:,2)),min(colbr(:,3))];
        hold on,
        hh=patch(xcoord,ycoord,colbrtot);
        set(hh,'EdgeColor',[0.8 0.8 0.8]);

    end;
    msp=msp+spacing;
end;
axis equal


